# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import pandas as pd
import traceback
from sklearn.preprocessing import OneHotEncoder
from ..systemlog import sysmanagerlog, syserrorlog
from ..ml_utils import runstatus


class XESOneHotEncoder(OneHotEncoder):
    """
    Feature selection, drop unused features and feature label.
    Encode categorical integer features using a one-hot aka one-of-K scheme.

    Parameters
    ----------
    data: pandas.DataFrame, default = None
        input data
    one_hot_file: configparser.ConfigParser, default = None
        config file, format: column:dims
    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import xes_onehot_encoder
    >>> import pandas as pd
    >>> import configparser
    >>> data = pd.read_csv("my_data.csv")
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> ins = xes_onehot_encoder.XESOneHotEncoder(data=data, conf=conf)
    >>> ins.init()
    >>> ins.execute()

    my_one_hot_file.conf
    --------
    subject_id:9  #Number of various values
    grade_id:13   #Number of various values
    """

    ONEHOT = "onehot"
    FIELDS = "fields"
    ONEHOT_FLAG = "onehot_flag"

    DEFAULT_DIM = -1

    def __init__(self, data=None, conf=None, log_path = None):
        super(XESOneHotEncoder, self).__init__()
        self._data = data
        self.xeasy_log_path = log_path
        self._one_hot_feature = {}
        self._conf = conf
        self._onehot_flag = False
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__, self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__, self.xeasy_log_path)

    def reset(self, data, one_hot_file):
        """
        Reset data.

        Parameters
        ----------
        data: pandas.DataFrame
            input data
        one_hot_file: ConfigParser.ConfigParsser
            config file, format: column: dims
        """
        if not isinstance(data, pd.DataFrame):
            self.managerlogger.logger.error("data is not dataframe")
            return False
        if one_hot_file == "":
            self.managerlogger.logger.error("one_hot_file is empty")
            return False
        self._data = data
        self._one_hot_file = one_hot_file

    def set_data(self, data):
        """
        Set data.

        Parameters
        ----------
        data: pd.DataFrame
        """
        if not isinstance(data, pd.DataFrame):
            self.managerlogger.logger.error("data is not dataframe")
            return False
        self._data = data

    def execute(self):
        """
        One hot encoder excute.
        """
        if self._onehot_flag:
            return self._xes_fit_transform_all()
        return False

    def get_data(self):
        """
        Get data.
        """
        return self._data

    def init(self):
        """
        read onehot configration

        Returns
        ----------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            # get conf
            try:
                self._onehot_flag = eval(self._conf.get(self.ONEHOT, self.ONEHOT_FLAG))
            except:
                self.errorlogger.logger.warning("no %s in config" % (self.ONEHOT))
                self._onehot_flag = False
            if not self._onehot_flag:
                return runstatus.RunStatus.SUCC
            onehot_field = self._conf.get(self.ONEHOT, self.FIELDS).replace(" ", "").split(",")

            # process onehot filed
            self._one_hot_feature = {}

            # init self._one_hot_feature
            if len(onehot_field) == 0:
                return runstatus.RunStatus.SUCC
            for field in onehot_field:
                field = field.split(":")
                if len(field) == 2:
                    self._one_hot_feature[field[0]] = int(field[1])
                elif len(field) == 1:
                    self._one_hot_feature[field[0]] = self.DEFAULT_DIM
            self.managerlogger.logger.info("onehot init succeed")
            return runstatus.RunStatus.SUCC
        except Exception as e:
            self.managerlogger.logger.error("onehot init error: %s" % e)
            self.errorlogger.logger.error("onehot init error: \n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _xes_fit_transform_all(self):
        """
        Fit and transform all features in configuration.

        Returns
        ----------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            for key in self._one_hot_feature:
                if key not in self._data.columns.tolist():
                    continue
                self._data = self._xes_fit_transform(self._data, key, self._one_hot_feature[key])
            self.managerlogger.logger.info("xes_fit_transform_all succeed")
            return True
        except Exception as e:
            self.managerlogger.logger.error("xes_fit_transform_all error: %s" % e)
            self.errorlogger.logger.error("xes_fit_transform_all error:\n %s" % traceback.format_exc())
            return False

    def _xes_fit_transform(self, train_data, field, field_dim):
        """
        Onehot encoder.

        Parameters
        ----------
        train_data: pandas.DataFrame
            train data
        field: str
            one hot field
        field_dim: onehot dims

        Returns
        ----------
        train data

        Examples
        ----------
        >>>df = pd.DataFrame([[1,2,1],[1,3,0,], [3,1,3]])
        >>>df.columns = ["f1", "f2", "f3"]
        >>>df
               f1  f2  f3
            0   1   2   1
            1   1   3   0
            2   3   1   3
        >>>self._xes_fit_transform(df, "f3", 3)
                    f1  f2  f3_1  f3_2  f3_3  f3_4
                0   1   2     0     1     0     0
                1   1   3     1     0     0     0
                2   3   1     0     0     0     1
        """

        try:
            # get default values
            sample_length = train_data.shape[0]

            if self.DEFAULT_DIM == field_dim:
                field_dim = int(max(train_data[field].tolist())) +1
            # Add one dimension for outlier handling.
            field_dim = field_dim +1
            new_feature = [[0 for x in range(sample_length)] for y in range(field_dim)]

            # xes onehot Compatible with negative numbers, all negative numbers and out-of-range
            # values will be classified as outliers and put into the same dimension.
            target = train_data[field].tolist()
            for i in range(sample_length):
                if (int(target[i]) >= 0) and (int(target[i]) < field_dim):
                    new_feature[int(target[i])][i] = 1
                else:
                    new_feature[field_dim - 1][i] = 1

            # insert new features of one hot
            for i in range(field_dim):
                train_data.insert(train_data.shape[1], field + "_" + str(i + 1), new_feature[i])

            # remove original feature
            del train_data[field]
            return train_data
        except:
            self.errorlogger.logger.error(traceback.format_exc())
            return train_data



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


from .base_model import SklearnModel
from sklearn.linear_model import LogisticRegression
import time
import traceback
from ..ml_utils import runstatus


class LR(SklearnModel):
    """A LogisticRegression classifier. The encapsulation form is sklearn model.

    Parameters
    --------
    config: configparser.ConfigParser()
        Configuration file for model initialization.

    Examples
    --------
    >>> from xes_ml_arch.src.model import lr
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> config = configparser.ConfigParser()
    >>> config.read("lr.conf")
    >>> x = pd.read_csv('data.txt', sep=',')
    >>> y = np.array([int(random.random() * 100) for _x in range(99)])
    >>> ins = lr.LR(config=config)
    >>> ins.init()
    >>> ins.train(x, y)
    ...
    ...
    """
    def __init__(self, config=None, log_path = None):
        SklearnModel.__init__(self, config=config, log_path = log_path)

    def _init_model(self):
        """Initialize model.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            self._model = LogisticRegression(**self._model_params)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("init lr model error: %s" % err)
            self.errorlogger.logger.error("init lr model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def train(self, x, y):
        """Train a model use a LogisticRegression model.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            sample data
        y: array-like of shape (n_samples,)
            Target vector relative to X.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            runstatus.RunStatus.Faild: Faild
        """
        try:
            t_start = time.time()
            self.managerlogger.logger.info("start lr..")
            self._model.fit(x, y)
            self.managerlogger.logger.info("finished lr!")
            t_end = time.time()
            self.managerlogger.logger.info("lr train time: %s" % (t_end - t_start))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("lr train error: %s " % err)
            self.errorlogger.logger.error("lr train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from sklearn.tree import DecisionTreeClassifier
from . import base_model
import traceback
from sklearn import tree
import pydotplus
from ..ml_utils import runstatus


class MyDesionTree(base_model.SklearnModel):
    """
        A decision tree classifier. The encapsulation form is sklearn model.

        Parameters
        --------
        config: configparser.ConfigParser()
            Configuration file for model initialization.

        Examples
        --------
        >>> from xes_ml_arch.src.model import desion_tree
        >>> import configparser
        >>> from sklearn.datasets import load_iris
        >>> config = configparser.ConfigParser()
        >>> config.read("desion_ress.conf")
        >>> clf = desion_tree.MyDesionTree(config=config)
        >>> iris = load_iris()
        >>> clf.train(iris.data, iris.target)
        ...
        ...
        """

    MODEL_ID = base_model.SklearnModel.MODEL_DST

    def __init__(self, config=None, log_path = None):
        super(MyDesionTree, self).__init__(config=config, log_path = log_path)

    def _init_model(self):
        """Initialize model.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            self._model = DecisionTreeClassifier(**self._model_params)
            return True
        except Exception as e:
            self.managerlogger.logger.error("init model error:\n %s" % e)
            self.errorlogger.logger.error("init model error:\n %s" % traceback.format_exc())
            return False

    def train(self, x, y):
        """Train a model use a decision tree.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            sample data
        y: array-like of shape (n_samples,)
            Target vector relative to X.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            self._model.fit(x, y)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("train model error: %s" % (err))
            self.errorlogger.logger.error("train model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def show_dot(self, file, feature):
        """Create decision tree visualization file, and save it as file.

        Parameters
        ------
        file: str
            file to store dot data and pdf
        feature: list
            List of feature names.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            runstatus.RunStatus.Faild: Faild
        """
        try:
            dot_data = tree.export_graphviz(self._model, feature_names=[str(x) for x in feature],
                                            class_names=[str(x) for x in self._model.classes_],
                                            out_file=None)

            # store bin tree, data an pdf
            with open(file + ".txt", "w") as f:
                f.write(dot_data)
            graph = pydotplus.graph_from_dot_data(dot_data)
            graph.write_pdf(file + ".pdf")
            self.managerlogger.logger.info("show dot succeed! ")
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("show dot error: %s" % err)
            self.errorlogger.logger.error("show dot error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def get_feature_importance(self, feature):
        """Get weights of user features.

        Parameters
        --------
        feature: list
            List of feature names.

        Returns
        --------
        res: dict
            Feature importance, like : dict([feature_importance, feature name])

        """
        try:
            res = zip(self._model.feature_importances_, feature)
            return res
        except Exception as e:
            self.managerlogger.logger.error("get_feature_importance error: %s" % e)
            self.errorlogger.logger.error('get_feature_importance error:\n %s' % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import pandas as pd
import configparser
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import data_sampler


class TestDataSampler(unittest.TestCase):
    def setUp(self):
        self._data_path = "../data/test.txt"
        self._config = "./conf/feature_enginnering.conf"
        self._data = pd.read_csv(self._data_path)
        self._conf = configparser.ConfigParser()
        self._conf_no = configparser.ConfigParser()
        self._conf_false = configparser.ConfigParser()
        self._conf.read(self._config)
        self._conf_no.read("./conf/feature_enginnering_no.conf")
        self._conf_false.read("./conf/feature_enginnering_false.conf")
        self.target = np.random.randint(0, 4, 100)
        # self._conf1.read("./conf/test_discretize_try.conf")
        self._ins = data_sampler.DataSampler(data=self._data,log_path = '../log/log.conf',conf=self._conf)
        self._ins_1 = data_sampler.DataSampler(data=None, log_path = '../log/log.conf',conf=self._conf)
        self._ins_2 = data_sampler.DataSampler(data=self._data, log_path = '../log/log.conf',conf=self._conf, target='col9')
        self._ins_3 = data_sampler.DataSampler(data=self._data, log_path = '../log/log.conf',conf=self._conf_no, target='col9')
        self._ins_4 = data_sampler.DataSampler(data=self._data, log_path = '../log/log.conf',conf=self._conf_false, target='col9')

    def test_excute(self):
        self.assertTrue(self._ins.init())
        self.assertFalse(self._ins.excute())

        self.assertTrue(self._ins_1.init())
        self.assertFalse(self._ins_1.excute())

        self.assertTrue(self._ins_2.init())
        self.assertTrue(self._ins_2.excute())

        self.assertFalse(self._ins_3.init())
        self.assertFalse(self._ins_3.excute())

        self.assertTrue(self._ins_4.init())
        self.assertTrue(self._ins_4.excute())

    def test_get_data(self):
        self._ins_2.init()
        self._ins_2.excute()
        data = self._ins_2.get_data()
        self.assertIsNotNone(data)

    def test_set_data(self):
        _data = None
        self._ins_2.init()
        self.assertTrue(self._ins_2.set_data(self._data))
        self.assertFalse(self._ins_2.set_data(_data))
        self.assertFalse(self._ins_2.set_data(self.target))

    def test_set_target_field(self):
        col = 1
        self.assertTrue(self._ins_2.set_target_field('col1'))
        self.assertFalse(self._ins_2.set_target_field(col))


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import SklearnModel
from sklearn.ensemble import RandomForestClassifier
import time
import traceback
from ..ml_utils import runstatus


class RF(SklearnModel):
    """A RandomForestClassifier classifier.
       Encapsulation form sklearn model

       Parameters
       ----------
       config: the instance of ConfigParser.ConfigParser().

       Examples
       ---------
       >>> from xes_ml_arch.src.model import rf
       >>> import configparser
       >>> import pandas as pd
       >>> import numpy as np
       >>> import random
       >>> config = configparser.ConfigParser()
       >>> config.read("rf.conf")
       >>> x = pd.read_csv('data.txt', sep=',')
       >>> y = np.array([int(random.random() * 100) for _x in range(99)])
       >>> ins = rf.RF(config=config)
       >>> ins.init()
       >>> ins.train(x, y)
       ...
       ...
    """

    def __init__(self, config=None, log_path = None):
        super(RF, self).__init__(config=config, log_path = log_path)
        self._n_estimators = 200
        self._max_feature = "auto"
        self._max_deepth = None

    def _init_model(self):
        """Init model.

        Returns
        -------
        Boolean: True(train successed) or False(train faild)
        """

        try:
            self._model = RandomForestClassifier(**self._model_params)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("init rf model error: %s" % err)
            self.errorlogger.logger.error("init rf model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def train(self, x, y):
        """Model train func.

        Parameters
        ----------
        x: input data.
        y: label.

        Returns
        -------
        Boolean: True(train suceessed) or False(train faild)
        """

        try:
            t_start = time.time()
            self.managerlogger.logger.info("start RF..")
            self._model.fit(x, y)
            self.managerlogger.logger.info("finished RF!")
            t_end = time.time()
            self.managerlogger.logger.info("RF train time: %s" % (t_end - t_start))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("rf train error: %s" % err)
            self.errorlogger.logger.error("rf train error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def get_feature_importance(self, feature):
        """Get feature weights of user data.

        Parameters
        ----------
        feature: Target characteristics(string of list).

        Returns
        ---------
        The score of feature importrance.(list, [[feature, score]]).
        """

        try:
            res = zip(self._model.feature_importances_, feature)
            return res
        except Exception as err:
            self.managerlogger.logger.error("rf get feature importance error: %s" % err)
            self.errorlogger.logger.error("rf get feature importance error:\n %s" % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
"""
@version: 1.0
@author: Jia Yanduo
@contact: jiayanduo@100tal.com
@site:
@software: PyCharm
@file: lightgbm.py
@time: 2019/11/15 13:50
"""

import time
import traceback
import numpy as np
import gc
from ..ml_utils import runstatus
try:
    import configparser
    from .base_model import BaseModel
except:
    import ConfigParser as configparser
    from base_model import BaseModel
try:
    import lightgbm as lgb
except:
    pass


class Lgbcf(BaseModel):
    GBM_N_ESTIMATORS = "num_boost_round"
    MODEL_ID = BaseModel.MODEL_CATE_LIGHTGBM

    def __init__(self, config=None, log_path = None):
        super(Lgbcf, self).__init__(config=config, log_path = log_path)
        self._lgb_param_ = []
        self.cate = []
        self.cf = False

    def _init_model(self):
        u'''
        init model
        :return:
        '''
        self._init_config()
        if 'categorical_feature' in self._model_params.keys():
            if len(self._model_params['categorical_feature']) > 0:
                self.cate = self._model_params['categorical_feature'].copy()
                self.cf = True
            del self._model_params['categorical_feature']
        # self._model = lgb.LGBMClassifier(**self._model_params)

        gc.collect()

    def _init_config(self):
        pass

    def train(self, x, y):
        u'''
        model train
        :param x: sample data
        :param y: label
        :return: bool
            True: suceess
            False: faild
        '''
        try:
            t_start = time.time()
            self.managerlogger.logger.info("start lightGBM..")
            if type(x) == tuple:
                if self.cf and set(self.cate) <= set(x[0].columns):
                    train_data = lgb.Dataset(x[0], label = y[0], categorical_feature = self.cate, free_raw_data = False)
                    valid_data = lgb.Dataset(x[1], label = y[1], categorical_feature = self.cate, free_raw_data = False)
                    self._model = lgb.train(self._model_params, train_data, valid_sets = valid_data,
                                    early_stopping_rounds = 100, verbose_eval = 100)
                else:
                    train_data = lgb.Dataset(x[0], label=y[0])
                    valid_data = lgb.Dataset(x[1], label=y[1])
                    self._model = lgb.train(self._model_params, train_data, valid_sets = valid_data,
                              early_stopping_rounds=100, verbose_eval = 100)
            else:
                if self.cf and set(self.cate) <= set(x.columns):
                    train_data = lgb.Dataset(x, label = y, categorical_feature = self.cate, free_raw_data=False)
                    self._model = lgb.train(self._model_params, train_data, valid_sets = train_data,
                                            early_stopping_rounds=100, verbose_eval = 100)
                else:
                    train_data = lgb.Dataset(x, label = y)
                    self._model = lgb.train(self._model_params, train_data, valid_sets = train_data,
                                            early_stopping_rounds=100, verbose_eval = 100)

            t_end = time.time()
            self.managerlogger.logger.info("finished lightGBM!")
            self.managerlogger.logger.info("lightGBM train time: %s" % (t_end - t_start))

            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("lightGBM train error: %s " % err)
            self.errorlogger.logger.error("lightGBM train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def predict(self, x):
        '''
        calculate data predict value
        :param x: sample data
        :param thresh: label classification threshold
        :return: result list, prediction label
        '''
        if self._model is None:
            return None
        #x_data = lgb.Dataset(x)
        thresh = 0.5
        res = self._model.predict(x)
        res=[0 if x < thresh else 1 for x in res]
        return np.array(res)

    def predict_proba(self, x):
        '''
        calculate data predict probability value
        :param x: sample data
        :return: list of probability value [[ ]]
        '''
        if self._model is None:
            return None
        # x_data = lgb.Dataset(x)
        res = self._model.predict(x)
        return res

    def get_feature_importance(self, feature):
        '''
        Get feature weights of user data
        :return: type:list, [[feature, score]]
        '''
        try:
            # feature_importance = []
            # importance = self._model.get_score(importance_type='gain')
            # for key in importance:
            #     feature_importance.append([importance[key], key])
            res = list(zip(self._model.feature_importance(importance_type='split'), self._model.feature_name()))
            #res = zip(self._model.feature_importance(), feature)
            return res

        except Exception as err:
            self.managerlogger.logger.error("lightGBM get feature importance error: %s" % err)
            self.errorlogger.logger.error("lightGBM get feature importance error:\n %s" % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


RES_PATH = ''

class Global(object):
    """Current package global parameters"""

    COLON = ":"
    FILE_READ = "rb"
    FILE_WRITE = "wb"
    TAB = "\t"
    COMMA = ","
    TIME_FORM = "%Y-%m-%d %H:%M:%S"
    global RES_PATH


# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import os
import shutil
import traceback
import configparser




class CreatNewDemo(object):
    """
    project_init
    when add a new project, run this to create a new project dir under demo
    """

    XES_FILE = 'xes_ml_arch'
    CONF_DIR = 'copy_dir'
    CONF_FILE = 'copy_file'
    PROJ = 'project'
    NEW_DIR = 'new_dir'

    def __init__(self, config, pro_path):
        """
        init config
        :param config: configure file
            copy_dir = config
            copy_file = __main__.py
            new_dir = data,result
        """

        self._model = None
        self._is_change = False
        self.pro_path = pro_path
        self._config = configparser.ConfigParser()
        self._config.read(config)

    def _create_new_dir(self, dir_name):
        """
        create a new dir under demo for a new project
        :param dir_name: string
            the name of new dir
        :return:
        """

        try:
            _dir = self._config.get(dir_name, self.CONF_DIR).replace(' ', '').split(',')
            _file = self._config.get(dir_name, self.CONF_FILE).replace(' ', '').split(',')
            _new = self._config.get(dir_name, self.NEW_DIR).replace(' ', '').split(',')
            self._path = os.path.dirname(__file__)
            file_path = self._path[:-13]
            # file_path = os.path.join(self._path, "monkey")
            _xes_dir = os.path.join(file_path, self.XES_FILE)
            # _src_dir = os.path.join(_xes_dir, self.SRC)
            _proj_dir = os.path.join(self.pro_path, self.PROJ)
            if not os.path.exists(_proj_dir):
                os.mkdir(_proj_dir)
            _new_dir = os.path.join(_proj_dir, dir_name)
            if not os.path.exists(_new_dir):
                os.mkdir(_new_dir)
                for mk in _new:
                    if not os.path.exists(os.path.join(_new_dir, mk)):
                        os.mkdir(os.path.join(_new_dir, mk))
            else:
                print("%s dir already exists. if you want to init this project, delete it frist" % (
                    _new_dir))
                return False

            for j in range(0, len(_file)):
                _tar_file = os.path.join(_new_dir, _file[j])
                shutil.copy(os.path.join(_xes_dir, _file[j]), _tar_file)

            for i in range(0, len(_dir)):
                _copy_dir = os.path.join(_xes_dir, _dir[i])
                _new_dir = os.path.join(_proj_dir, dir_name)
                _tar_dir = os.path.join(_new_dir, _dir[i])

                if os.path.exists(_copy_dir) and not os.path.exists(_tar_dir):
                    shutil.copytree(_copy_dir, _tar_dir)
                else:
                    print("{} is not exist! or new demo dir {} is exist! ".format(_xes_dir,_tar_dir))
            return True
        except:
            print("create new demo {} not succeed".format(dir_name))
            traceback.print_exc()
            return False

    def start(self):
        """
        project init start, create new dir based config file
        :return:
        """

        for key in self._config.sections():
            name = key.replace(' ', '')
            if self._create_new_dir(name):
                print("%s project init succeed" % name)
            else:
                print("%s project init failed" % name)

def create_project(pro_path):
    path = os.path.dirname(__file__)
    conf = os.path.join(path, 'create_demo.conf')
    ins = CreatNewDemo(conf, pro_path)
    ins.start()
    print("Succ init project.")

if __name__ == '__main__':
    conf = './create_demo.conf'
    #ins = CreatNewDemo(conf)
    #ins.start()
    pro_path = os.getcwd()
    create_project(pro_path)


# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT
from .base_model import BaseModel
import time
import traceback
from ..ml_utils import runstatus

try:
    from xgboost import XGBRegressor
except:
    pass
    # raise ImportError("no model named xgboost")


class SklearnXGBReg(BaseModel):
    """A xgboost regress.
       Encapsulation form base model

       Parameters
       ----------
       config: the instance of ConfigParser.ConfigParser()

       Examples
       --------
       >>> from xes_ml_arch.src.model import sklearn_xgb_reg
       >>> import configparser
       >>> import pandas as pd
       >>> import numpy as np
       >>> import random
       >>> config = configparser.ConfigParser()
       >>> config.read("sklearn_xgb.conf")
       >>> x = pd.read_csv('data.txt', sep=',')
       >>> y = np.array([int(random.random() * 100) for _x in range(99)]) #label = num of samples
       >>> ins = sklearn_xgb.SklearnXGB(config=config)
       >>> ins.init()
       >>> ins.train(x, y)
       ...
       ...
    """

    MODEL_ID = BaseModel.MODEL_SKLEARN_XGB_REG

    def __init__(self, config=None, log_path = None):
        super(SklearnXGBReg, self).__init__(config=config, log_path = log_path)

    def _init_model(self):
        """Init mode"""
        self._model = XGBRegressor(**self._model_params)
        print(self._model_params)

    def train(self, x, y):
        """Model train function.

        Parameters
        ----------
        x: input sample data.
        y: label.

        Returns
        ---------
        Bool: True(train successed) or False(train faild).
        """
        try:
            t_start = time.time()
            self.managerlogger.logger.info("start xgboost.sklearn..")
            self._model.fit(x, y)
            self.managerlogger.logger.info("finished xgboost.sklearn!")
            t_end = time.time()
            self.managerlogger.logger.info("xgboost.sklearn train time: %s" % (t_end - t_start))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("xgboost.sklearn train error: %s " % err)
            self.errorlogger.logger.error("xgboost.sklearn train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def predict(self, x, thresh=0.5):
        """Forecast input data.

        Parameters
        ----------
        x：input sample data.
        thresh: label classification threshold.

        Returns
        -------
        Prediction label(list).
        """

        try:
            return self._model.predict(x)
        except Exception as err:
            self.managerlogger.logger.error("xgboost.sklearn predict error: %s " % err)
            self.errorlogger.logger.error("xgboost.sklearn predict error:\n %s " % traceback.format_exc())
            return None

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        ----------
        x: input sample data.

        Returns
        ---------
        list of probability value [[ ]]
        """
        try:
            return self._model.predict(x)
        except Exception as err:
            self.managerlogger.logger.error("xgboost.sklearn predict_proba error: %s " % err)
            self.errorlogger.logger.error("xgboost.sklearn predict_proba error:\n %s " % traceback.format_exc())
            return None

    def get_feature_importance(self, feature):
        """Get feature weights of user data.

        Parameters
        ----------
        feature: Target characteristics(string of list).

        Returns
        ---------
        if faild return None,else return the score of feature importrance.(list, [[feature, score]]).
        """
        try:
            res = zip(self._model.feature_importances_, feature)
            return res
        except Exception as err:
            self.managerlogger.logger.error("xgboost.sklearn get_feature_importance error: %s " % err)
            self.errorlogger.logger.error(
                "xgboost.sklearn get_feature_importance error:\n %s " % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import configparser
import pandas as pd
import pickle

import sys
from xgboost import XGBClassifier
# import xes_ml_arch as xml
import xeasy_ml as xml
import warnings

warnings.filterwarnings('ignore')

sys.path.append("../../")

BASE_CONFIG = "base_config"
RESULT_PATH = "result_path"
data_file_name = "./data/test.txt"

def predict(conf = './config/demo/feature_enginnering.conf', model_path = "./result/v1/model/demo.m", model_name='sklearn_xgb'):
    '''

    Parameters
    ----------
    conf: str
        Config file path for feature_enginnering.conf.
    model_path: str
        The storage path of the trained model.
    model_name：str
        Name of your model.

    Returns
    -------
    predict_res: list
        Predict results.
    '''
    print("start predict...")
    # Load feature processing configuration file.
    ml_config = configparser.ConfigParser()
    ml_config.read(conf)

    # Initialize the prediction class.
    xeasy_log_path = None
    ml = xml.prediction_ml.PredictionML(xeasy_log_path=xeasy_log_path)

    # Load model
    if model_name == 'sklearn_xgb':
        ml._model = XGBClassifier()
        ml._model.load_model(model_path)
    else:
        ml._model = pickle.load(open(model_path, xml.global_pre.Global.FILE_READ))

    # Load data
    ml.set_data(pd.read_csv(data_file_name))

    # Data preprocessing
    ml._feature_processor = xml.data_processor.DataProcessor(conf=ml_config)
    ml._feature_processor.init()
    ml._feature_processor.test_data = ml._data
    ml._feature_processor.execute()

    # Test data
    test_feature = ml._feature_processor.test_data_feature
    test_feature = test_feature.astype("float64", errors='ignore')

    # Predict result
    predict_res = ml._model.predict(test_feature)

    return predict_res

def train():
    print("start ml...")
    ml_config = configparser.ConfigParser()
    ml_config.read("./config/demo/ml.conf")
    xml.global_pre.RES_PATH = ml_config.get(BASE_CONFIG, RESULT_PATH)

    xeasy_log_path = None

    ml = xml.train_model_ml.TrainML(conf=ml_config, xeasy_log_path=xeasy_log_path)

    if ml.init() == xml.runstatus.RunStatus.FAILED:
        print("ml init failed")
    if ml.set_data(pd.read_csv("./data/sample.txt")) == xml.runstatus.RunStatus.FAILED:
        print("ml set data failed")
    if ml.start() == xml.runstatus.RunStatus.FAILED:
        print("ml train failed")
    print("finish ml!")


if __name__ == '__main__':
    train()
    result = predict()
    print(result)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import pandas as pd
import configparser
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import data_washer


class TestDataSampler(unittest.TestCase):
    def setUp(self):
        self._data_path = "../data/test.txt"
        self._config = "./conf/feature_enginnering.conf"
        self._data = pd.read_csv(self._data_path)
        self._conf = configparser.ConfigParser()
        self._conf_no = configparser.ConfigParser()
        self._conf_false = configparser.ConfigParser()
        self._conf.read(self._config)
        self._conf_no.read("./conf/feature_enginnering_no.conf")
        self._conf_false.read("./conf/feature_enginnering_false.conf")
        # self._conf1.read("./conf/test_discretize_try.conf")
        self._ins = data_washer.DataWasher(data=self._data, log_path = '../log/log.conf',conf=self._conf)
        self._ins_1 = data_washer.DataWasher(data=None, log_path = '../log/log.conf',conf=self._conf)

    def test_start(self):
        self.assertTrue(self._ins.init())
        self._ins.set_train_flag()
        self._ins.excute()
        self.assertTrue(self._ins_1.init())
        self._ins_1.set_data(self._data)
        self._ins.set_train_flag()
        self.assertIsNotNone(self._ins.excute())

    def test_get_data(self):
        self._ins.init()
        data = self._ins.get_data()

    def test_set_data(self):
        self._ins.init()
        self._ins.set_data(self._data)

    def test_set_flag(self):
        self._ins.init()
        self._ins.set_train_flag()
        self._ins.set_test_flag()

    def test_store_zscore_scale(self):
        self._ins._store_zscore_scale()

    def test_control_data(self):
        self._ins.control_data({"col1": None})

        self._ins.control_data({"col1": "fun_add_self"})

if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from . import base_model
from . import lr
from . import my_xgb
from . import rf
from . import linear
from . import sklearn_xgb
from . import sklearn_xgb_reg
from . import desion_tree
from . import my_lightgbm
from . import lgb_category
import configparser
from ..ml_utils import runstatus


class ModelFactory(object):
    """Model factory.Create a model based on the parameters passed in.

    Parameters
    --------
    config: configparser.ConfigParser()
         Configuration file of model.
    model_name: str
         Name of model.

    Examples
    --------
    >>> from xes_ml_arch.src.model import model_factory
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> config = configparser.ConfigParser()
    >>> config.read("model_online.conf")
    >>> x = pd.read_csv('data.txt', sep=',')
    >>> y = np.array([int(random.random() * 100) for _x in range(99)])
    >>> ins = model_factory.ModelFactory()
    >>> my_model = ins.create_model(config=config, model_name='lr')
    >>> my_model.train(x, y)
    ...
    ...
    """
    @staticmethod
    def create_model(config, model_name=None,log_path = None):
        """Create a model.

        Parameters
        --------
        config: configparser.ConfigParser
            Config object.
        model_name: str
            Name of model.

        Returns
        --------
        :return: model object or None

        Notes
        --------
        This function may raise exception include TypeError, RuntimeError
        """
        model = None

        # get model name
        if not isinstance(config, configparser.ConfigParser):
            raise TypeError("config object is not instance of ConfigParser")

        if model_name is None:
            try:
                model_name = config.get(base_model.BaseModel.BASE_CONF,
                                        base_model.BaseModel.MODEL_NAME)
            except configparser.Error:
                raise RuntimeError("config has no section named %s, or has no option named %s" % (
                    base_model.BaseModel.BASE_CONF, base_model.BaseModel.MODEL_NAME))

        # create a model
        if model_name == base_model.BaseModel.MODEL_XGB:
            model = my_xgb.MyXgb(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_LR:
            model = lr.LR(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_RF:
            model = rf.RF(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_LINE_REG:
            model = linear.Liner(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_SKLEARN_XGB:
            model = sklearn_xgb.SklearnXGB(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_SKLEARN_XGB_REG:
            model = sklearn_xgb_reg.SklearnXGBReg(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_DST:
            model = desion_tree.MyDesionTree(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_LIGHTGBM:
            model = my_lightgbm.MyLightGBM(config=config,log_path = log_path)
        elif model_name == base_model.BaseModel.MODEL_CATE_LIGHTGBM:
            model = lgb_category.Lgbcf(config=config, log_path = log_path)
        else:
            pass

        if model is None:
            raise RuntimeError("can not create a model named: %s" % (model_name))

        # Initialize model
        if runstatus.RunStatus.FAILED == model.init():
            raise RuntimeError("model init faild")

        return model



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from sklearn.linear_model import LogisticRegression
import pickle
import os
import traceback
from ..ml_utils import runstatus
from ..ml_utils import global_pre
from ..systemlog import sysmanagerlog, syserrorlog

try:
    import xgboost as xgb
except:
    pass


class BaseModel(object):
    """This basic class encapsulates the methods of model definition, model training, model prediction,
        model saving, and model loading, etc.

    Parameters
    --------
    config: configparser.ConfigParser(), default = None
        Model configuration file.

    Attributes
    ----------
    _model : ml model object
        used which model

    _model_params : json or dict = {key: word}
        used model params.
            eg.{
                 "bst:max_depth": 10,
                 "bst:eta": 1,
                 "silent": 1,
                 "objective":"binary:logistic",
                 "learning_rate": 0.1,
                 "n_estimators": 3000,
                 "num_boost_round": 100,
                 "min_child_weight": 1,
                 "subsample": 0.9,
                 "colsample_bytree": 0.9,
                 "nthread": 4
                }

    _config : ConfigParser.ConfigParser(),
        The config of model.

    _is_trained : bool, True or False
        flag of train

    _mode_file : file with path
        store trained model to _mode_file

    _feature_weight_file : int
        The number of outputs when ``fit`` is performed.

    _log : system log

    _logerror: error debug log
    """

    MODEL_XGB = "xgb"
    MODEL_SKLEARN_XGB = "sklearn_xgb"
    MODEL_LIGHTGBM = "lightgbm"
    MODEL_CATE_LIGHTGBM = 'lightgbm_cf'
    MODEL_SKLEARN_XGB_REG = "sklearn_xgb_reg"
    MODEL_SVM = "svm"
    MODEL_SVR = "svr"
    MODEL_SCV = "svc"
    MODEL_RF = "rf"
    MODEL_LR = "lr"
    MODEL_DST = "dst"
    MODEL_GBDT = "gbdt"
    MODEL_LINE_REG = "line_reg"
    MODEL_PARAM = "train_param"
    MODEL_NAME = "model_name"
    MODEL_FILE = "model_file"
    FEATURE_WEIGHT_FILE = "feature_weight_file"
    BASE_CONF = "base_conf"
    PARAMS = "params"

    MODEL_ID = MODEL_LR

    def __init__(self, config=None, log_path = None):
        # used model
        self._model = None
        # flag of train
        self._is_trained = False
        # config
        self._config = config
        # model params
        self._model_params = {}

        self.xeasy_log_path = log_path
        # model file
        self._mode_file = None

        # feature importance
        self._feature_weight_file = None

        # system log
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def _init_model(self):
        """Define the initial model, default LR."""

        self._model = LogisticRegression()

    def init(self):
        """Initialize model, enclude init params and new model object.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            self._init_params()
            self._init_model()
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("init model error: %s" % err)
            self.errorlogger.logger.error("init model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def train(self, x, y):
        """Model train, call the methods in the subclass.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            sample data
        y: array-like of shape (n_samples,)
            Target vector relative to X.
        """

        self._is_trained = True

    def predict(self, x):
        """Data predict, call the methods in the subclass, if subclass don't have methods, run this

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            test data.

        Returns
        --------
        :return: predict results
        """
        pass

    def predict_proba(self, x):
        """Calculate data predict probability value
        You can call the methods in the subclass, if subclass don't have methods, run this function.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            test data

        Returns
        --------
        :return: predict probability value
        """
        pass

    def get_feature_importance(self, feature):
        """Get user feature weights.
        You can call the methods in the subclass, if subclass don't have methods, run this.

        Parameters
        --------
        feature: list
            All features list.

        Returns
        --------
        :return: list, [[feature, score]]
        """
        pass

    def store_model(self, model_file=None):
        """Store model file.

        Parameters
        --------
        model_file: str
            Name of stored model file.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            if model_file is None:
                model_file = self._mode_file
            if isinstance(self._model, xgb.sklearn.XGBClassifier):
                self._model.save_model(self._mode_file)
            else:
                pickle.dump(self._model, open(model_file, global_pre.Global.FILE_WRITE))
            self.managerlogger.logger.info("store model succeed!")
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("store model error: %s" % err)
            self.errorlogger.logger.error("store model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def load_model(self, model_file=None):
        """Load model from model file.

        Parameters
        --------
        model_file: str
            Name of exist model file.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            if model_file is None:
                model_file = self._mode_file
            if isinstance(self._model, xgb.sklearn.XGBClassifier):
                self._model.load_model(self._mode_file)
            else:
                self._model = pickle.load(open(model_file, global_pre.Global.FILE_READ))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("load model error: %s" % err)
            self.errorlogger.logger.error("load model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def store_feature_importance(self, feature):
        """Store feature importance to local file.

        Parameters
        --------
        feature: list
            List of feature_name.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        score = self.get_feature_importance(feature)
        try:
            score = sorted(score, key=lambda x: x[0], reverse=True)
            with open(self._feature_weight_file, "w") as file_handle:
                for data in score:
                    file_handle.write(str(data[1]) + "\t" + str(data[0]) + "\n")
            self.managerlogger.logger.info("store_feature_importance succeed")
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("store_feature_importance error: %s" % err)
            self.errorlogger.logger.error("store_feature_importance error:\n %s" % traceback.format_exc())
        return runstatus.RunStatus.FAILED

    def _init_params(self):
        """Initialize model params, get model params from config file."""

        # if not config model params, use default
        if self.MODEL_ID not in self._config.sections():
            self._model_params = {}
            self.managerlogger.logger.info("%s model use default params")
        else:
            self._model_params = eval(self._config.get(self.MODEL_ID, self.PARAMS))

        self._mode_file = self._config.get(self.BASE_CONF, self.MODEL_FILE)
        self._feature_weight_file = self._config.get(self.BASE_CONF, self.FEATURE_WEIGHT_FILE)
        self._mode_file = os.path.join(global_pre.RES_PATH, self._mode_file)
        self._feature_weight_file = os.path.join(global_pre.RES_PATH, self._feature_weight_file)
        if not os.path.isdir(os.path.dirname(self._mode_file)):
            os.mkdir(os.path.dirname(self._mode_file))
        if not os.path.isdir(os.path.dirname(self._feature_weight_file)):
            os.mkdir(os.path.dirname(self._feature_weight_file))


class SklearnModel(BaseModel):
    """A model based on SklearnModel.

    Parameters
    ----------
    config: configparser.ConfigParser()
        config should contains model param, otherwise use the default
    """

    def __init__(self, config=None, log_path = None):
        super(SklearnModel, self).__init__(config=config, log_path = log_path)

    def predict(self, x):
        """Calculate data predict result.

        Parameters
        --------
        x: pandas.DataFrame with shape (n_samples, n_features)
            Features of test data.

        Returns
        --------
        :return: array like
            Predict result.
        """
        # if self._model is not None:
        try:
            return self._model.predict(x)
        except:
            self.errorlogger.logger.error("predict error:\n %s" % traceback.format_exc())
            return None

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        --------
        x: pandas.DataFrame with shape (n_samples, n_features)
            Features of test data.

        Returns
        --------
        res: list
            List of probability value belonging to a certain class.
        """
        # if self._model is not None:
        try:
            res = self._model.predict_proba(x)
            # return [x[1] for x in res]
            return res
        except:
            self.errorlogger.logger.error('predict_proba error:\n %s' % traceback.format_exc())
            return None

    def get_feature_importance(self, feature):
        """Get weights of user feature.

        Parameters
        --------
        feature: list
            List of feature_name.

        Returns
        --------
        res: list
            Feature importance list [[feature, score]].
        """
        try:
            res = zip(self._model.coef_[0], feature)
            return res
        except Exception as e:
            self.managerlogger.logger.error("get_feature_importance error:\n %s" % e)
            self.errorlogger.logger.error("get_feature_importance error:\n %s " % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import configparser
import traceback
import json
import pandas as pd
from sklearn.preprocessing import LabelEncoder
from ..systemlog import sysmanagerlog
from ..systemlog import syserrorlog
from ..ml_utils import runstatus
from ..ml_utils import pre_utils
from ..ml_utils import global_pre
import pickle
import numpy as np
import os


class FeatureFilter(object):
    """
    Base class for feature filter that includes feature selection, unused features droping
    and feature label.
    Includefeature selection, drop unused features and feature label.

    Parameters
    ----------
    conf: configparser.ConfigParser()
        config
    data_file: str
        data file path
    del_list: list
        columns should be delete
    del_keywords: list
        columns should be delete which contains key word in `del_keywords`
    label_list: list
        columns which should be labeled
    selected_list: list
        columns which should be selected
    data: pandas.DataFrame
        samples

    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import feature_filter
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> data = pd.read_csv("my_data.csv")
    # this
    >>> ins = feature_filter.FeatureFilter(conf=conf)
    >>> ins.init()
    >>> ins.set_data(data)
    >>> ins.excute()
    # or this
    >>> ins = feature_filter.FeatureFilter(conf=conf, data=data)
    >>> ins.init()
    >>> ins.excute()
    # or this
    >>> ins = feature_filter.FeatureFilter(conf=conf, data_file="my_data.csv")
    >>> ins.init()
    >>> ins.excute()
    """
    PREPERTIES_FILE = "./config/label.properties"
    FEATURA_FILTER = "feature_filter"
    DELETE_FIELD_LIST = "delete_field_list"
    DELETE_KEYWORDS = "delete_keyword_list"
    LABEL_LIST = "label_list"
    SELECTED_LIST = "selected_list"

    ENCODEE_FILE_PATH = "label_encoder_file"
    ENCODER_TABLE = ".table"

    def __init__(self, conf=None, log_path = None, data_file=str(), del_list=list(), del_keywords=list(),
                 label_list=list(), selected_list=list(), data=None):
        # conf
        self._conf = conf
        self.xeasy_log_path = log_path
        # data to filter
        self._data = data
        # data file, if data is None ,read_data from file
        self._data_file = data_file
        # fields to delete
        self._del_list = del_list
        # fields which contain keywords to delete
        self._del_keywords = del_keywords
        # fields to label
        self._label_list = label_list
        # fields to select
        self._extract_list = selected_list
        # data columns
        self._columns = []

        # feature label encoder
        self._feature_lebel_encoder = None
        self._feature_lebel_encoder_file = ""
        self._feature_lebel_encoder_table_file = ""

        # logs
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Read information from the configuration file for initialization.
        Returns
        ---------
        :return: bool
        """
        try:
            if not isinstance(self._conf, configparser.ConfigParser):
                self.managerlogger.logger.error("init feature_filter error: config is None")
                return runstatus.RunStatus.FAILED

            # get fields from conf
            self._del_list = self._get_list_from_conf(self.DELETE_FIELD_LIST)
            self._del_keywords = self._get_list_from_conf(self.DELETE_KEYWORDS)
            self._label_list = self._get_list_from_conf(self.LABEL_LIST)
            self._extract_list = self._get_list_from_conf(self.SELECTED_LIST)

            # load encoder
            self._load_encoder_obj()

            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("feature filter init error: %s" % err)
            self.errorlogger.logger.error("feature filter init error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def set_data(self, data):
        """
        set data

        Parameters
        ----------
        data: data to filter, type: pandas.DataFrame
            Initial data for training or testing.

        Returns
        ----------
        :return: bool, runstatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: False
        """
        if not pre_utils.PredictUtils.valid_pandas_data(data):
            self.managerlogger.logger.error("feature filter set data error")
            return runstatus.RunStatus.FAILED
        self._data = data
        return runstatus.RunStatus.SUCC

    def excute(self):
        """
        The basic feature filtering methods include lowercase column names, feature cleaning, feature
        extraction, feature deletion by column or keyword, and target tag encoding.

        Returns:
        ------
        self._data
        """
        try:
            # Convert column names into lists and convert all column names to lowercase.
            self._init()
            # Clean the features
            self.clean_features()
            # Extract features
            self.extract_features()
            # Delete features not used by colunms.
            self.drop_features()
            # Delete fields which contains the keywords.
            self.drop_features_by_keywords()
            # Encode target labels with value between 0 and n_classes-1.
            self.label_fields()
            return self._data
        except Exception as ex:
            self.errorlogger.logger.error("feature filter excute error:\n %s" % traceback.format_exc())
            self.managerlogger.logger.error("feature filter excute error: %s" % ex)

    def get_data(self):
        """
        Get filted data
        Returns
        ----------
        self._data
        """
        return self._data

    def reset(self, data=None, del_list=None, del_keywords=None, label_list=None,
              selected_list=None):
        """
        Set data, del_list, del_keywords, etc.

        Parameters
        ----------
        data: pandas.DataFrame
            orignal data
        del_list: list
            columns need to be deleted
        del_keyword: list
            key words of the columns need to be deleted
        label_list: list
            columns need to be labeled
        """
        if data is not None:
            self._data = data
            self._columns = self._data.columns
        if del_list is not None:
            self._del_list = del_list
        if del_keywords is not None:
            self._del_keywords = del_keywords
        if label_list is not None:
            self._label_list = label_list
        if selected_list is not None:
            self._extract_list = selected_list

    def clean_features(self, empty_val=-1):
        """Clean the features, currently including the replacement of outliers, the filling of
        missing values, and the deletion of duplicate columns."""

        # replace exception
        self._data = self._data.replace("\\N", 0)
        # fill null
        self._data = self._data.fillna(empty_val)
        self._data.columns = self._data.columns.str.strip(" ")

        # remove duplicated columns
        self._data = self._data.loc[:, ~self._data.columns.duplicated()]
        self._columns = self._data.columns.tolist()

    def drop_features(self):
        """
        Delete fields not used. This function will update self._data, self._columns.

        Examples
        --------
        Given a dataset with three features and four samples, we let the drop_features.
        >>>_del_list=["feature1"]
        >>>df = pd.DataFrame([[0, 0, 3], [1, 1, 0], [0, 2, 1], [1, 0, 2]])
        >>>df.columns=["feature1","feature2","feature3"]
        ['feature1' 'feature2' 'feature3']
        >>>drop_features()
        ['feature2' 'feature3']
        """

        # warning: filter function return object in python 3.x
        self._del_list = list(filter(None, self._del_list))
        if len(self._del_list) == 0:
            return
        # case insensitive
        self._del_list = self.case_insensitive(self._del_list)
        self._columns = self.case_insensitive(self._columns)

        del_list = self.intersect_cloumns(self._del_list, self._columns)

        if len(del_list) > 0:
            # Delete by column.
            self._data = self._data.drop(list(del_list), 1)
        self._columns = self._data.columns.tolist()

    def drop_features_by_keywords(self):
        """
        Delete fields which contains the keywords. This function will update self._data.

        Examples
        --------
        Given a dataset with three features and four samples, we let the drop_fields.
        >>>_del_list=["testfeature"]
        >>>df = pd.DataFrame([[0, 0, 3], [1, 1, 0], [0, 2, 1], [1, 0, 2]])
        >>>df.columns=["testfeature1","testfeature2","feature3"]
        ['feature1' 'feature2' 'feature3']
        >>>drop_features_by_keywords()
        ['feature3']
        """
        self._del_keywords = filter(None, self._del_keywords)
        #  case insensitive
        self._del_keywords = self.case_insensitive(self._del_keywords)
        self._columns = self.case_insensitive(self._columns)
        del_list = []
        for _col in self._columns:
            for _keyword in self._del_keywords:
                if _col.find(_keyword) != -1:
                    del_list.append(_col)
        if len(del_list) == 0:
            return

        self._data = self._data.drop(list(del_list), 1)
        self._columns = self._data.columns.tolist()

    def label_fields(self):
        '''
        Label some field.
        Encode target labels with value between 0 and n_classes-1.
        '''
        if len(self._label_list) == 0:
            return

        res = {}
        encoder_flag = False

        if not self._feature_lebel_encoder:
            self._feature_lebel_encoder = {}
            encoder_flag = True

        try:

            for _f in self._label_list:
                if _f not in self._columns:
                    continue

                # After adding labels that need to be labeled, you need to store the data.
                if _f not in self._feature_lebel_encoder and not encoder_flag:
                    encoder_flag = True

                if encoder_flag:
                    lebel_encoder = LabelEncoder()
                    # Fit label encoder and return encoded labels.
                    self._data[_f] = [x + 1 for x in lebel_encoder.fit_transform(self._data[_f])]
                    res[_f] = dict(zip(lebel_encoder.classes_.tolist(),
                                       range(1, len(lebel_encoder.classes_.tolist()) + 1)))
                    self._feature_lebel_encoder[_f] = lebel_encoder
                else:
                    lebel_encoder = self._feature_lebel_encoder.get(_f)
                    try:
                        self._data[_f] = [x + 1 for x in lebel_encoder.transform(self._data[_f])]
                    except:
                        class_arr = lebel_encoder.classes_.tolist()
                        try:
                            tmp_label_val = np.searchsorted(class_arr, self._data[_f].tolist(),
                                                            sorter=np.argsort(class_arr))

                            self._data[_f] = [x + 1 for x in tmp_label_val]
                        except:
                            continue

            if encoder_flag:
                self._store_label_properties(res)

        except Exception as e:
            self.errorlogger.logger.error("label_fields error:\n %s " % traceback.format_exc())
            self.managerlogger.logger.error("label_fields error:\n %s " % e)

    def extract_features(self):
        """
        Extract features. This function will update self._data and self._columns.

        Examples
        --------
        Given a dataset with three features and four samples, we let the extract_features.
        >>>_del_list=["feature1"]
        >>>df = pd.DataFrame([[0, 0, 3], [1, 1, 0], [0, 2, 1], [1, 0, 2]])
        >>>df.columns=["feature1","feature2","feature3"]
        ['feature1' 'feature2' 'feature3']
        >>>extract_feature()
        ['feature1']
        """

        try:
            self._extract_list = list(filter(None, self._extract_list))
            if len(self._extract_list) == 0:
                return
            # case insensitive
            self._extract_list = self.case_insensitive(self._extract_list)
            self._columns = self.case_insensitive(self._columns)

            extract_list = self.intersect_cloumns(self._extract_list, self._columns)

            # if selected_list is empty，skip this function
            if len(extract_list) == 0:
                return
            self._data = self._data[list(extract_list)]
            self._columns = self._data.columns.tolist()
        except Exception as ex:
            self.errorlogger.logger.error("extract_features error:\n %s " % traceback.format_exc())
            self.managerlogger.logger.error("extract_features error:\n %s " % ex)

    def _store_label_properties(self, res):
        """
        Store label properties.

        Parameters
        ----------
        res: dict
            result of filter

        """
        try:
            self._valid_path(self._feature_lebel_encoder_table_file)
            self._valid_path(self._feature_lebel_encoder_file)

            res = map(lambda x: "%s = %s" % (x[0], json.dumps(x[1])), res.items())
            with open(self._feature_lebel_encoder_table_file, "w") as file:
                file.write("\n".join(res))

            pickle.dump(self._feature_lebel_encoder, open(self._feature_lebel_encoder_file, "wb"))
        except Exception as e:
            self.managerlogger.logger.warning("store label properties error: %s" % e)

    def _init(self):
        # Convert column names into lists and convert all column names to lowercase.
        if self._data is None:
            self._data = pd.read_csv(self._data_file, sep=",", encoding="utf-8",
                                     error_bad_lines=False, low_memory=True)
        if not isinstance(self._data, pd.DataFrame):
            raise TypeError
        # case insensitive, using lowercase by default
        self._data.columns = self.case_insensitive(self._data.columns)
        self._columns = self._data.columns.tolist()

    def case_insensitive(self, list):
        """
        Parameters
        ----------
        data: list
            list need to be converted to lowercase

        Returns
        ----------
        :return: lowercase list
        """
        return [x.lower() for x in list]

    def intersect_cloumns(self, list1, list2):
        """
        Parameters
        ----------
        list1: list need intersected
        list2: list need intersected

        Returns
        ----------
        resultList: list
            sorted list
        """
        set1 = set(list1)
        set2 = set(list2)

        resultList = list(set1.intersection(set2))
        resultList.sort()
        return resultList

    def _get_list_from_conf(self, option):
        """Read options from conf, and replace ' ', split by comma"""
        res = list()
        if option in self._conf.options(self.FEATURA_FILTER):
            keyword_str = self._conf.get(self.FEATURA_FILTER, option).lower()
            res = keyword_str.replace(" ", "").split(global_pre.Global.COMMA)
        return res

    def _load_encoder_obj(self):
        """Read label encoding information from configuration file."""
        try:
            self._feature_lebel_encoder_file = self._conf.get(self.FEATURA_FILTER,
                                                              self.ENCODEE_FILE_PATH)
            self._feature_lebel_encoder_table_file = "%s%s" % (
                self._feature_lebel_encoder_file, self.ENCODER_TABLE)
            self._feature_lebel_encoder = pickle.load(open(self._feature_lebel_encoder_file, "rb"))
        except:
            self._feature_lebel_encoder = None
            self.errorlogger.logger.warning("load encoder error")

    def _valid_path(self, file_path):
        """
        Verify that the file exists.

        Parameters
        ----------
        file_path : str
        """
        if not file_path:
            raise ValueError("file path is invalid: %s" % (file_path))
        path, file_name = os.path.split(file_path)

        if not os.path.isdir(path):
            os.makedirs(path)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


from collections import Iterable

from sklearn import metrics
from sklearn.preprocessing import label_binarize
import pandas as pd
import numpy as np


class PredictUtils(object):
    """Model prediction Class Initialization."""

    @staticmethod
    def store_feature_importance(result, store_file):
        """Store feature weight to file and sort.

        Parameters
        ----------
        result: Feature weights(float of list).
        store_file: path of store

        Returns
        -------
        sort of result.
        """
        if result is None:
            raise AttributeError("get feature weight exception: result is None")
        result.sort(key=lambda x: x[0], reverse=True)
        with open(store_file, "w") as file_handle:
            for data in result:
                file_handle.write(str(data[1]) + "\t" + str(data[0]) + "\n")
        return result

    @staticmethod
    def get_precision(label, pre_label):
        """Precision prediction score.

        In prediction, this function computes subset precision:
        the set of labels predicted for a sample must *exactly* match the corresponding set of labels in true.

        Parameters
        ----------
        label: 1d array-like, or label indicator array / sparse matrix Ground truth (correct) labels.
        pre_label: 1d array-like, or label indicator array / sparse matrix Predicted labels, as returned by a classifier.

        Returns
        -------
        Jaccard Distance score : float.
        """

        if not isinstance(label, list):
            try:
                label = list(label)
            except Exception:
                raise TypeError("label type is %s" % (type(label)))

        if not isinstance(pre_label, Iterable):
            try:
                pre_label = list(pre_label)
            except Exception:
                raise TypeError("pre_label type is %s" % (type(pre_label)))

        if len(label) != len(pre_label):
            raise ValueError("length of label and pre_label is not equal")

        return metrics.jaccard_similarity_score(label, pre_label)

    @staticmethod
    def get_model_score(target_true, target_pred):
        """Model evaluation index;Different indicators can measure the model from different perspectives.

        Parameters
        ----------
        target_true: 1d array-like, or label indicator array / sparse matrix Ground truth (correct) labels in test.
        target_pred: 1d array-like, or label indicator array / sparse matrix Predicted labels, as returned by a classifier in test.

        Returns
        -------
        Array:[Confusion metrics,Recall,Precision,F1-socre,auc]
        """

        if not isinstance(target_true, list):
            try:
                target_true = list(target_true)
            except Exception:
                raise TypeError("label type is %s" % (type(target_true)))

        if not isinstance(target_pred, list):
            try:
                target_pred = list(target_pred)
            except Exception:
                raise TypeError("pre_label type is %s" % (type(target_pred)))

        if len(target_true) != len(target_pred):
            raise ValueError("length of label and pre_label is not equal")

        try:
            confuseion = metrics.confusion_matrix(target_true, target_pred)

            # judge is binary-class or multiclass
            # if only a few samples, maybe all the labels are 0/1
            if len(set(target_true)) <= 2:
                class_type = "binary"
            else:
                class_type = "macro"
            recall = metrics.recall_score(target_true, target_pred, average=class_type)
            precision = metrics.precision_score(target_true, target_pred, average=class_type)
            f1_score = metrics.f1_score(target_true, target_pred, average=class_type)
            # jaccard = metrics.jaccard_similarity_score(target_true, target_pred)
            jaccard = metrics.jaccard_score(target_true, target_pred, average="weighted")
        except:
            confuseion = None
            recall = None
            precision=None
            f1_score=None
            jaccard=None

        try:
            msr = metrics.mean_squared_error(target_true, target_pred)
        except:
            msr = None

        res = dict(confuseion=confuseion, recall=recall, precision=precision, f1_score=f1_score,
                   jaccard=jaccard, msr=msr)

        return res


    @staticmethod
    def get_roc_score(label, prob_label):
        """Get mode auc in dateset.AUC (area under curve) is defined as the area under the ROC curve. We often use AUC
        value as the evaluation standard of the model, because often the ROC curve can not clearly indicate which
        classifier is better, but as a value, the classifier with larger AUC is better.

        Parameters
        ----------
        label: 1d array-like, or label indicator array / sparse matrix Ground truth (correct) labels.
        target_pred: 1d array-like, or label indicator array / sparse matrix Predicted labels, as returned by a classifier in test.
        prob_label

        Returns
        -------
        AUC_socre: float
        """

        try:
            if not isinstance(label, list):
                try:
                    label = list(label)
                except Exception:
                    raise TypeError("label type is %s" % (type(label)))

            if not isinstance(prob_label, list):
                try:
                    prob_label = list(prob_label)
                except Exception:
                    raise TypeError("pre_label type is %s" % (type(prob_label)))

            if len(label) != len(prob_label):
                raise ValueError("length of label and pre_label is not equal")

            if not isinstance(prob_label[0],Iterable):
                try:
                    prob_label = list(map(lambda x: np.array([1.0-abs(x),abs(x)]), prob_label))
                except Exception:
                    raise TypeError("pre_label type is %s" % (type(prob_label)))


            n_class = len(set(label))
            # if only a few samples, maybe all the labels are 0/1, then len(set(label)) ==1 ，
            #  this may occur exception in roc_auc_score. Set label[0] as a different value to avoid.
            if n_class == 1:
                label[0] = 1.0 - label[0]
                n_class = len(set(label))
            label_bin = label_binarize(label, classes=list(set(label)))

            if len(prob_label[0]) != n_class:
                raise ValueError(
                    "class num is %s, but prob dimensionality is %s" % (
                        n_class, len(prob_label[0])))

            # judge is binary-class or multiclass
            if n_class == 2:
                auc = metrics.roc_auc_score(label_bin, [x[1] for x in prob_label])
            else:
                auc = metrics.roc_auc_score(label_bin, prob_label)

            return dict(auc=auc)
        except:
            return dict(auc=-1)

    @staticmethod
    def valid_pandas_data(data):
        """Valid data type detection, is pandas DataFrame, shape is not zero.

        Parameters
        ----------
        data: input date;

        Returns
        -------
        Boolean: if input date is Dataframe and shape is not zero then True else False.
        """

        if data is None:
            return False
        if not isinstance(data, pd.DataFrame):
            return False
        if data.shape[0] == 0:
            return False
        return True



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import configparser
import random
import pandas as pd
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.analysis import analysis
from xes_ml_arch.src.cross_validation import cross_validation
from xes_ml_arch.src.ml import train_model_ml
from xes_ml_arch.src.ml_utils import runstatus


class TestAnalysis(unittest.TestCase):
    def setUp(self):
        self.conf = configparser.ConfigParser()
        self.conf_try = configparser.ConfigParser()
        self.conf_no = configparser.ConfigParser()
        self.conf_try1 = configparser.ConfigParser()
        self.conf_try2 = configparser.ConfigParser()
        self.conf_try3 = configparser.ConfigParser()

        self.conf_file = "./config/ml.conf"
        self.conf.read(self.conf_file)
        self.conf_try.read("./config/ml_try.conf")
        self.conf_no.read("./conf/ml_online.conf")

        self.conf_try1.read("./config/ml_try1.conf")
        self.conf_try2.read("./config/ml_try2.conf")
        self.conf_try3.read("./config/ml_try3.conf")

        self.columns = ["col%s" % x for x in range(10)]
        # self.x = self.columns[:9]
        # self.y = self.columns[9]
        self.get_data()
        self.ins = train_model_ml.TrainML(conf=self.conf,xeasy_log_path = '../log/log.conf')
        self.ins_try = train_model_ml.TrainML(conf=self.conf_try,xeasy_log_path = '../log/log.conf')
        self.ins_no = train_model_ml.TrainML(conf=self.conf_no,xeasy_log_path = '../log/log.conf')
        self.ins_try1 = train_model_ml.TrainML(conf=self.conf_try1,xeasy_log_path = '../log/log.conf')
        self.ins_try2 = train_model_ml.TrainML(conf=self.conf_try2,xeasy_log_path = '../log/log.conf')
        self.ins_try3 = train_model_ml.TrainML(conf=self.conf_try3,xeasy_log_path = '../log/log.conf')

    def test_set_data(self):
        self.assertTrue(self.ins.set_data(self._data))
        self.assertTrue(self.ins_try.set_data(self._data))

    def test_start(self):
        if self.ins.init() == runstatus.RunStatus.SUCC:
            self.assertTrue(self.ins.set_data(self._data))
            self.assertTrue(self.ins.start())
            self.assertTrue(self.ins.set_data(self._data_no))
            self.assertFalse(self.ins.start())
            self.assertTrue(self.ins.set_data(self._data_tt))
            self.assertFalse(self.ins.start())
        self.assertFalse(self.ins_try.init())
        self.assertFalse(self.ins_try.start())
        self.assertFalse(self.ins_no.init())
        self.assertFalse(self.ins_no.start())

    def test_init(self):
        assert (self.ins.init(), runstatus.RunStatus.SUCC)
        self.assertFalse(self.ins_try1.init())
        self.assertFalse(self.ins_try1.start())
        self.assertFalse(self.ins_try2.init())
        self.assertFalse(self.ins_try2.start())
        self.assertFalse(self.ins_try3.init())
        self.assertFalse(self.ins_try3.start())

    def test_set_analysis_ins(self):
        ana = analysis.Analysis(log_path=  '../log/log.conf')
        noa = None
        self.ins.set_analysis_ins(ana)
        try:
            self.ins.set_analysis_ins(noa)
        except:
            pass

    def test_set_cross_validation(self):
        cv = cross_validation.Cross_Validation(log_path=  '../log/log.conf')
        nocv = None
        self.ins.set_cross_validation(cv)
        try:
            self.ins.set_cross_validation(nocv)
        except:
            pass

    def test_init_cv_ins(self):
        self.assertTrue(self.ins._init_cv_ins())
        self.assertFalse(self.ins_no._init_cv_ins())

    def test_train_handle(self):
        self.ins = train_model_ml.TrainML(conf=self.conf,xeasy_log_path = '../log/log.conf')
        self.assertTrue(self.ins.init())
        self.assertTrue(self.ins.set_data(self._data))
        self.assertTrue(self.ins.start())

    def get_data(self):
        self._data = pd.DataFrame(
            [[int(random.random() * 100) for x in range(10)] for y in range(10000)],
            columns=self.columns)
        self._data["col9"] = np.random.choice([0, 1], size=10000)
        self._data_no = pd.DataFrame(
            [[int(random.random() * 100) for x in range(10)] for y in range(10000)],
            columns=self.columns)
        self._data_tt = pd.DataFrame(
            [[int(random.random() * 100) for x in range(4)] for y in range(10000)],
            columns=['col1', 'col2', 'col3', 'col4'])


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from ..systemlog import sysmanagerlog, syserrorlog
from ..ml_utils import runstatus
from ..ml_utils import global_pre
import configparser
import traceback
import pandas as pd
from ..ml_utils import pre_utils
from sklearn.model_selection import train_test_split, GridSearchCV, RandomizedSearchCV
try:
    from xgboost import XGBClassifier
except:
    pass

class Optimizing():

    OPTIMIZING_CONF = "optimizing_conf"
    ENABLE_GRIDSEARCH = "enable_gridSearch"
    CV = "cv"
    N_JOBS = "n_jobs"

    def __init__(self,config = None,log_path = None):
        # config
        self._conf= config
        self.xeasy_log_path = log_path
        # model params
        self._cv = 5
        self._n_jobs = 1
        self._model_params = {}
        self._estimator = None
        self._enable_gridsearch = None
        self._best_params = None
        self.best_estimator_ = None
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__, self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__, self.xeasy_log_path)

    def init(self):
        """Read optimizer parameter configuration information

        Returns
        -------
        Boolean: if "enable_gridSearch","cv", "n_jobs" exists return runstatus.RunStatus.SUCC
                otherwise runstatus.RunStatus.FAILED.
        """
        if not isinstance(self._conf, configparser.ConfigParser):
            self.managerlogger.logger.error("conf error: conf is not ConfigParser instance")
            config = configparser.ConfigParser()
            config.read(self.default_config_file)
            self._conf = config
        try:
            self._enable_gridsearch = eval(self._conf.get(self.OPTIMIZING_CONF, self.ENABLE_GRIDSEARCH))
            self._cv = eval(self._conf.get(self.OPTIMIZING_CONF, self.CV))
            self._n_jobs = eval(self._conf.get(self.OPTIMIZING_CONF, self.N_JOBS))
            return runstatus.RunStatus.SUCC
        except Exception as ex:
            self.managerlogger.logger.error("optimizing object init erorr: %s" % ex)
            self.errorlogger.logger.error("optimizing object init erorr \n" + traceback.format_exc())
        return runstatus.RunStatus.FAILED



    def excute(self, estimator = None, params = None, x = None, y = None ):
        """Model training search for the best params.

        Parameters
        ----------
        estimator: base model.
        params: parameter list which to search.
        x: input data.
        y: label.

        Returns
        -------
        The best value of the parameter in the parameter list.
        """

        self._estimator = estimator._model
        self._model_params = params
        try:
            grid = RandomizedSearchCV(self._estimator,self._model_params, cv=self._cv, n_jobs=self._n_jobs)
            grid.fit(x,y)
            if grid:
                self._best_params = grid.best_params_
                self.best_estimator_ = grid.best_estimator_
                self.managerlogger.logger.info("optimizing excute succeed! ")
                self.managerlogger.logger.info(grid.best_params_)
                return runstatus.RunStatus.SUCC
            else:
                self.managerlogger.logger.error("optimizing excute error! ")
                return runstatus.RunStatus.FAILED
        except Exception as ex:
            self.managerlogger.logger.error("optimizing excute erorr: %s" % ex)
            self.errorlogger.logger.error("optimizing excute erorr \n" + traceback.format_exc())
        return runstatus.RunStatus.FAILED



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import itertools
import pandas as pd
from ..ml_utils import runstatus, global_pre
from ..systemlog import sysmanagerlog, syserrorlog
import traceback
import matplotlib.pyplot as plt
import os
import numpy as np
from functools import reduce
import warnings

warnings.filterwarnings("ignore")


class Analysis(object):
    """
    This class is an analysiser to analysis a data set. You can call various methods below the
    class for visual display and analysis of data set.

    Parameters
    --------
    conf : configparser.ConfigParser, default = None
        config of analysis
    data : pandas.DataFrame, default = None
        data set
    feature_columns : list, default = None
        column name of feature vector. It is a feature list to be analysis.
    label_columns : str, default = ""
        label column name

    Examples
    --------
    >>> from xes_ml_arch.src.analysis import analysis
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> columns = ["col%s" % x for x in range(10)]
    >>> x = columns[:9]
    >>> y = columns[9]
    >>> data = pd.DataFrame(
    >>>           [[int(random.random() * 100) for _x in range(10)] for _y in range(10000)], columns=columns)
    >>> data["col9"] = np.random.choice([0, 1], size=10000)
    # this
    >>> ins = analysis.Analysis(conf=conf, data=data, feature_columns=x,
    >>>                        label_columns=y)
    >>> ins.execute()
    # or this
    >>> ins = analysis.Analysis(conf=conf)
    >>> ins.reset(conf=conf, data=data, feature_columns=x,
    >>>                        label_columns=y)
    >>> ins.execute()
    """
    ANALYSIS = "analysis"
    FEATURE_DEL_THRESH = "feature_del_thresh"
    ANALYSIS_RESULT = "analysis_result"
    DELETED_FILE = "deleted_feature.txt"
    BOX = "box"
    HIST = "hist"
    RELATION = "relation"
    NAME_DICT_FILE = "name_dict_file"
    FEATURE_WITH_FEATURE = "feature_with_feature"
    FEATURE_WITH_LABEL = "feature_with_label"

    THRESHOLD_DEFAULT = 0.9

    def __init__(self, conf=None, log_path = None, data=None, feature_columns=None, label_columns=""):
        self._data = data

        self.log_path = log_path

        self._conf = conf
        self._feature_columns = feature_columns
        self._label_column = label_columns
        self._feature_corr = []
        self._target_corr = []
        self._result_path = None

        self.managerlogger = sysmanagerlog.SysManagerLog(__file__, self.log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.log_path)

    def reset(self, conf=None, data=None, feature_columns=None, label_columns=""):
        """
        Set conf, data, feature_columns, label_columns.
        If value is default, then pass.

        Parameters
        --------
        conf : configparser.ConfigParser, default = None
            config of analysis
        data : pandas.DataFrame, default = None
            data set
        feature_columns : list, default = None
            column name of feature vector. It is a feature list to be analysis.
        label_columns : str, default = ""
            label column name
        """
        if conf is not None:
            self._conf = conf
        if data is not None:
            self._data = data
        if feature_columns is not None:
            self._feature_columns = feature_columns
        if label_columns != "":
            self._label_column = label_columns

    def execute(self):
        """
        Start to analysis.

        Returns
        --------
        :return: bool, runstatus.RunStatus
             runstatus.RunStatus.SUCC: True
             runstatus.RunStatus.FAILED: Failed
        """
        try:
            # self._init()
            # get feature correlation and save as document
            self.get_feature_corr()
            # get correlation between features and label, and save as document
            self.get_label_corr()
            # Delete one of the two features whose correlation is higher than the threshold, and save
            # the deleted feature name to the file
            self.get_del_feature()
            # plot box for every feature
            self.plot_box()
            # plot hist figure for every feture
            self.plot_hist()
            self.managerlogger.logger.info("analysis execute succeed!")
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.errorlogger.logger.error("analysis execute error! \n" + traceback.format_exc())
            self.managerlogger.logger.error("analysis execute error!  %s" % err)
            return runstatus.RunStatus.FAILED

    def get_feature_corr(self):
        """
        Get correlation between x1 ang x2 in X, and save as document.
        """
        self._checkout()
        feature_combin = itertools.combinations(self._feature_columns, 2)
        self._feature_corr = []
        for x1, x2 in feature_combin:
            key = (x1, x2)
            if not x1 in self._data.columns or not x2 in self._data.columns:
                continue
            self._feature_corr.append([key, self._data[x1].corr(self._data[x2])])

        # sort and store feature's correlation
        self._feature_corr.sort(key=lambda x: x[1], reverse=True)
        with open(os.path.join(self._result_path, self.FEATURE_WITH_FEATURE), "w") as file_handle:
            file_handle.write(
                "\n".join(["%s:%s %s" % (x[0][0], x[0][1], x[1]) for x in self._feature_corr]))

    def get_label_corr(self):
        """Get correlation between features(x) and label(y), and save as document."""
        self._checkout()
        self._target_corr = []
        for column in self._feature_columns:
            if column not in self._data.columns:
                continue
            corration = self._data[column].corr(self._data[self._label_column])
            corration = 0 if np.isnan(corration) else corration
            self._target_corr.append([column, corration])

        # sort and store correlation between feature and label
        self._target_corr.sort(key=lambda x: abs(float(x[1])), reverse=True)
        with open(os.path.join(self._result_path, self.FEATURE_WITH_LABEL), "w") as file_handle:
            file_handle.write("\n".join(["%s:%s" % (x[0], float(x[1])) for x in self._target_corr]))

    def get_del_feature(self):
        """When correaltion between x1 and x2 beyond a threshold, keep x2 and drop x1."""

        if len(self._feature_corr) == 0:
            raise ValueError("correlation is between x and y is empty")
        try:
            threshold = float(self._conf.get(self.ANALYSIS, self.FEATURE_DEL_THRESH))
        except Exception as err:
            self.managerlogger.logger.error("threshold is not is config, use default threshold value. %s " % err)
            self.errorlogger.logger.error("get_del_feature error:\n %s " % traceback.format_exc())
            threshold = self.THRESHOLD_DEFAULT

        last_result = filter(lambda x: x[1] > threshold, self._feature_corr)
        del_id_list = [x[0][0] for x in last_result]
        with open(os.path.join(self._result_path, self.DELETED_FILE), "w") as file_handle:
            file_handle.write("\n".join(del_id_list))

    def plot_box(self):
        """Plot boxplots for every feature."""

        self._checkout()
        box_png = os.path.join(self._result_path, self.BOX)
        if not os.path.isdir(box_png):
            os.mkdir(box_png)
        for feature in self._feature_columns:
            if feature in self._data.columns and feature != self._label_column:
                # reload(sys)
                # sys.setdefaultencoding('utf8') #python3 no need?
                self._data[[feature, self._label_column]].boxplot(by=self._label_column)
                plt.savefig(os.path.join(box_png, u"%s.png" % (feature)))
                plt.close()

    def plot_hist(self):
        """Plot histogram."""

        self._checkout()
        # Init path to save png.
        hist_png_path = os.path.join(self._result_path, self.HIST)
        if not os.path.isdir(hist_png_path):
            os.mkdir(hist_png_path)
        label_set = set(self._data[self._label_column].tolist())

        # plot png for every feture
        for feature in self._feature_columns:
            if feature in self._data.columns:
                plot_dict = {}
                for label in label_set:
                    plot_dict[label] = self._data[self._data[self._label_column] == label][
                        feature].tolist()
                # reload(sys)
                # sys.setdefaultencoding('utf8')
                # plot histogram
                self._plot_data_n(plot_dict, os.path.join(hist_png_path, u"%s.png" % (feature)))
                # plot distribut of all different target
                self._plot_distrib(plot_dict,
                                   os.path.join(hist_png_path, u"%s_dis.png" % (feature)))
                # plot statistic for one feature data
                self._plot_relation(plot_dict,
                                    os.path.join(hist_png_path, u"%s_rel.png" % (feature)))
                # draw a line graph of the relationship between feature values and labels
                self._plot_relation_2(plot_dict,
                                      os.path.join(hist_png_path, u"%s_rel_2.png" % (feature)))

    def _plot_relation(self, plot_dict, res_path):
        """
        Plot statistic for one feature data.

        Parameters
        --------
        plot_dict: dict
            {key:value},key is a value in set(data[target]), value is list of data[feature]
        res_path: str
            store path
        """
        all_values = reduce(lambda x, y: x + y, [x[1] for x in plot_dict.items()])
        value_list = list(set(all_values))
        value_list.sort()
        plt.figure(figsize=(15, 10))

        dict_vlaues_all = dict([[x, all_values.count(x)] for x in value_list])

        for key in plot_dict:
            tmp_list = [float(plot_dict[key].count(x)) / dict_vlaues_all[x] for x in value_list]
            plt.plot(value_list, tmp_list, label=str(key))
        plt.legend()
        plt.savefig(res_path)
        plt.close()

    def _plot_relation_2(self, plot_dict, res_path):
        """
        Draw a line graph of the relationship between feature values and labels.

        Parameters
        --------
        plot_dict: dict
            {key:value},key is a value in set(data[target]), value is list of data[feature]
        res_path: str
            store path
        """
        all_values = reduce(lambda x, y: x + y, [x[1] for x in plot_dict.items()])
        value_list = list(set(all_values))
        value_list.sort()
        count_value = [all_values.count(x) for x in value_list]
        sum_value = [sum(count_value[:i + 1]) for i in range(len(count_value))]
        dict_vlaues_all = dict(zip(value_list, sum_value))
        plt.figure(figsize=(15, 10))

        for key in plot_dict:
            tmp_count = [plot_dict[key].count(x) for x in value_list]
            tmp_sum = [sum(tmp_count[:i + 1]) for i in range(len(tmp_count))]
            tmp_list = [float(tmp_sum[i]) / dict_vlaues_all[x] for i, x in enumerate(value_list)]
            plt.plot(value_list, tmp_list, label=str(key))
        plt.legend()
        plt.savefig(res_path)
        plt.close()

    def _plot_distrib(self, plot_dict, res_path):
        """
        Plot distribut of all different target.

        Parameters
        --------
        plot_dict: dict
            {key:value},key is a value in set(data[target]), value is list of data[feature]
        res_path: str
            store path
        """
        all_values = reduce(lambda x, y: x + y, [x[1] for x in plot_dict.items()])
        value_list = list(set(all_values))
        value_list.sort()
        x_lim = range(len(value_list))
        width = 0.3

        plt.figure(figsize=(15, 10))
        plt.xticks(x_lim, [str(_) for _ in value_list])
        for key in plot_dict:
            tmp_list = [plot_dict[key].count(x) for x in value_list]
            plt.bar(x_lim, tmp_list, width=width, label=str(key))
            x_lim = [_ + width for _ in x_lim]
        plt.legend()
        plt.savefig(res_path)
        plt.close()

    def _plot_data_n(self, plot_dict, res_path):
        """
        Plot hist figure.

        Parameters
        --------
        plot_dict: dict
            {key:value},key is a value in set(data[target]), value is list of data[feature]
        res_path: str
            store path
        """
        # all_values = reduce(lambda x, y: x + y, plot_dict)
        #
        all_values = reduce(lambda x, y: x + y, [x[1] for x in plot_dict.items()])
        plt.xlim([min(all_values) - 0.1, max(all_values) + 0.1])
        # plt.bar()
        plt.figure(figsize=(15, 10))

        # Judging whether it is classification or regression.
        if len(plot_dict) > 20:
            tmp_value = len(set(all_values))
            bins = int(min(max(tmp_value / 10, 10), 100))
            plt.hist(all_values, bins, alpha=0.3, label=str("all"))
        else:
            for key in plot_dict:
                tmp_value = len(set(plot_dict[key]))
                bins = int(min(max(tmp_value / 10, 10), 100))
                plt.hist(plot_dict[key], bins, alpha=0.3, label=str(key))

        plt.legend(loc='upper right')
        plt.title(res_path)
        plt.savefig(res_path)
        plt.close()

    def _checkout(self):
        """
        checkout data, feature and label

        Notes
        --------
        If you do not meet the specification, exception is thrown.
        """
        if self._data is None:
            raise ValueError("data is None")
        if not isinstance(self._data, pd.DataFrame):
            raise TypeError("data type is not pandas.DataFrame")
        if self._feature_columns is None or self._label_column == "":
            raise ValueError("sample feature or label is empty")

        if not self._label_column in self._data.columns:
            raise AttributeError("data not contains %s, contained fields is [%s]" % (
                self._label_column, str(self._data.columns.tolist())))

        self._result_path = global_pre.RES_PATH
        # unittest
        #self._result_path = self._conf.get(self.ANALYSIS, self.ANALYSIS_RESULT)

        if not os.path.isdir(self._result_path):
            os.mkdir(self._result_path)

    def _init(self, data):
        pass



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import pandas as pd
import configparser
import numpy as np
import sys
import json

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import pre_feature_utils


class TestPreFeatureUtils(unittest.TestCase):
    def setUp(self):
        self.conf_1 = configparser.ConfigParser()
        self.conf = configparser.ConfigParser()
        self.conf_d = configparser.ConfigParser()
        self.ff_t = pre_feature_utils.PreFeatureUtils(conf=self.conf_1,log_path = '../log/log.conf')
        self.conf.read('./conf/feature_enginnering.conf')
        self.conf_d.read('./conf/feature_enginnering_no.conf')
        self.ff = pre_feature_utils.PreFeatureUtils(conf=self.conf,log_path = '../log/log.conf')
        self.ff_no = pre_feature_utils.PreFeatureUtils('./conf/feature_enginnering.conf',log_path = '../log/log.conf')
        self.data = pd.read_csv('data/featureFilter.csv')
        self.ff_d = pre_feature_utils.PreFeatureUtils(conf=self.conf, data=self.data,log_path = '../log/log.conf')
        self.ff_f = pre_feature_utils.PreFeatureUtils(conf=self.conf_d,log_path = '../log/log.conf')

    def test_init(self):
        # app = self.conf.get('pre_feature_utils', "single_feature_apply")
        self.assertTrue(self.ff.init())
        self.assertFalse(self.ff_no.init())
        self.assertFalse(self.ff_t.init())

    def test_setdata(self):
        a = [1, 2, 3, 4]
        self.assertFalse(self.ff.set_data(a))

    def test_start(self):
        self.assertTrue(self.ff.init())
        self.assertTrue(self.ff.set_data(self.data))
        res = self.ff.excute()
        print("res:", res)

    def test_start_a(self):
        self.assertTrue(self.ff_d.init())
        res2 = self.ff_d.excute()
        print("res2:", res2)

    def test_start_b(self):
        self.assertTrue(self.ff_f.init())
        res_no = self.ff_f.excute()
        print("res_no:", res_no)

    def test_start_c(self):
        self.assertTrue(self.ff_f.set_data(self.data))
        res1 = self.ff_f.excute()
        print("res1:", res1)


if __name__ == '__main__':
    unittest.main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xeasy-ml",
    version="0.1.3",
    author="X",
    author_email="author@example.com",
    description="An integrated machine learning analysis framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src1"},
    packages=setuptools.find_packages(where="src1"),
    package_data={'xeasy_ml': ['project_init/*','xes_ml_arch/*','xes_ml_arch/config/*','xes_ml_arch/config/demo/*','xes_ml_arch/data/*','xes_ml_arch/log/*','xes_ml_arch/result/*',
                             'xes_ml_arch/src/tests/analysis_test/config/*','xes_ml_arch/src/tests/analysis_test/log/*','xes_ml_arch/src/tests/analysis_test/*',
                             'xes_ml_arch/src/tests/config/*', 'xes_ml_arch/src/tests/cross_validation_test/*','xes_ml_arch/src/tests/cross_validation_test/model/*',
                             'xes_ml_arch/src/tests/data/*', 'xes_ml_arch/src/tests/feature_enginner_test/*','xes_ml_arch/src/tests/feature_enginner_test/conf/*',
                             'xes_ml_arch/src/tests/feature_enginner_test/config/*','xes_ml_arch/src/tests/feature_enginner_test/data/*','xes_ml_arch/src/tests/feature_enginner_test/model/*',
                             'xes_ml_arch/src/tests/ml_test/config/*', 'xes_ml_arch/src/tests/ml_tests/*', 'xes_ml_arch/src/tests/model/*','xes_ml_arch/src/tests/model_test/*',
                             'xes_ml_arch/src/tests/model_test/config/*', 'xes_ml_arch/src/tests/*']
                  },

    install_requires=[ 'scikit-learn==0.24.1',
                    'pandas==0.24.2',
                    'matplotlib==3.3.4',
                    'pydotplus==2.0.2',
                    'xgboost==1.4.2',
                    'numpy==1.19.5'
                ],
    python_requires=">=3.6",
)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


from xeasy_ml.project_init import create_new_demo
from xeasy_ml.xes_ml_arch.src.analysis import analysis
from xeasy_ml.xes_ml_arch.src.constance import predict_constance
from xeasy_ml.xes_ml_arch.src.cross_validation import cross_validation, data_split
from xeasy_ml.xes_ml_arch.src.feature_enginnering import data_processor, data_sampler, data_washer, feature_discretizer, feature_filter, pre_feature_utils, xes_onehot_encoder
from xeasy_ml.xes_ml_arch.src.ml import base_ml, prediction_ml, train_model_ml
from xeasy_ml.xes_ml_arch.src.ml_utils import configmanager, feature_processor, global_pre, jsonmanager, pre_utils, runstatus
from xeasy_ml.xes_ml_arch.src.model import base_model, desion_tree, linear, lr, model_factory, my_lightgbm, my_xgb, rf, sklearn_xgb, sklearn_xgb_reg, svr
from xeasy_ml.xes_ml_arch.src.optimizing import optimizing
from xeasy_ml.xes_ml_arch.src.systemlog import accesslog, syserrorlog, syslog, sysmanagerlog
from xeasy_ml.xes_ml_arch.src.schema import accessdictmodel, data_set




# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

class PredictConstance(object):
    """Constance of predict class."""

    BASE_CONFIG = "base_config"
    MODEL_NAME = "model_name"
    MODEL_CONFIG = "model_config"

    CV_CONFIG = "cv_config"

    FEATURE_ENGINEERING_CONFIG = "feature_engineering_config"

    ANALYSIS_CONFIG = "analysis_config"

    DATA_SPLITE_CONFIG = "data_split_config"

    RESULT_PATH = "result_path"
    PREDICT_FILE = "predict_result.txt"
    TEST_SCORE = "test_score.txt"

    PRE = "pre"
    PROBA = "proba"



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import pickle
import traceback
import os
import pandas as pd
from sklearn import preprocessing
from ..systemlog import sysmanagerlog
from ..systemlog import syserrorlog
from ..ml_utils import runstatus, global_pre


class DataWasher(object):
    """
        Sort column, zscore.

        Parameters
        ----------
        data : pandas.DataFrame, default = None
            data need wash
        conf : configparser.ConfigParser, default = None
        zscal_pickle_file : default = None
        train_flag : bool, default = False
            set train flag as true, train data need wash
        processor : default = None

        Examples
        --------
        >>> from xes_ml_arch.src.feature_enginnering import data_washer
        >>> import configparser
        >>> import pandas as pd
        >>> conf = configparser.ConfigParser()
        >>> conf.read("myconfig.conf")
        >>> data = pd.read_csv("my_data.csv")
        # this
        >>> ins = data_washer.DataWasher(data=data, conf=conf)
        >>> ins.init()
        >>> ins.set_train_flag()
        >>> # or ins.set_test_flag()
        >>> ins.excute()
        # or this
        >>> ins = data_washer.DataWasher(conf=conf)
        >>> ins.init()
        >>> ins.set_data(data)
        >>> ins.set_train_flag()
        >>> ins.excute()
        """
    MEAN = "mean"
    SCALE = "scale"
    PROPERTIES = ".properties"
    DATA_WASHER = "data_washer"
    ZSCAL_PICKLE_FILE = "zscal_pickle_file"
    WASH_FLAG = "wash_flag"

    def __init__(self, data=None, conf=None, log_path = None, zscal_pickle_file=None, train_flag=False,
                 processor=None):
        self._data = data
        self._conf = conf
        self.xeasy_log_path = log_path
        self._zscal_pickle_file = zscal_pickle_file
        self._train_flag = train_flag
        self._scale = None
        self._processer = processor
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Init data washer.

        Returns
        ----------
        :return: bool, runstatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            _zscal_pickle_file = self._conf.get(self.DATA_WASHER, self.ZSCAL_PICKLE_FILE)
            self._zscal_pickle_file = os.path.join(global_pre.RES_PATH, _zscal_pickle_file)
            if not os.path.isdir(os.path.dirname(self._zscal_pickle_file)):
                os.mkdir(os.path.dirname(self._zscal_pickle_file))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("data washer init error: %s" % err)
            self.errorlogger.logger.error("data washer init error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def get_data(self):
        """
        Get data.

        Returns
        ----------
        self._data
        """
        return self._data

    def set_data(self, data):
        """
        Set data need wash.

        Parameters
        ----------
        data: pandas.DataFrame
            data need wash.
        """
        self._data = data

    def set_train_flag(self):
        """
        Set train flag as true, train data need wash
        default is False
        """
        self._train_flag = True

    def set_test_flag(self):
        """
        Set _train_flag as false, data no need wash when test
        default is False

        """
        self._train_flag = False

    def excute(self):
        """
        starting wash data.

        Returns
        ----------
        self._data
        """
        try:
            self._init()
            self.sort_columns()

            if not self._wash_flag:
                self.managerlogger.logger.info("no need data wash")
                return self._data

            self.control_data(self._processer)
            self.zscore_data()
            self.managerlogger.logger.info("data washer excute succeed")
            return self._data
        except Exception as e:
            self.managerlogger.logger.error("data washer excute error: %s" % e)
            self.errorlogger.logger.error("data washer excute error:\n %s " % traceback.format_exc())
            return None

    def sort_columns(self):
        """Sort columns by default."""

        columns = self._data.columns.tolist()
        columns.sort()
        self._data = self._data[columns]

    def zscore_data(self):
        """
        Standardize func.
        self._data : the data to be scaled
        :return: data after scaling
        """
        if not isinstance(self._data, pd.DataFrame):
            raise TypeError

        colnames = self._data.columns.tolist()
        if self._train_flag:
            self._scale = self._scale.fit(self._data)
            pickle.dump(self._scale, open(self._zscal_pickle_file, "w"))
            self._store_zscore_scale()
        after_scale_data = self._scale.transform(self._data)
        self._data = pd.DataFrame(after_scale_data, columns=colnames)

    def _init(self):
        """Init param"""

        # load zscore file or excute standarScaler
        if self.WASH_FLAG in self._conf.options(self.DATA_WASHER):
            self._wash_flag = eval(self._conf.get(self.DATA_WASHER, self.WASH_FLAG))
        if not self._wash_flag:
            return
        if not self._train_flag:
            try:
                self._scale = pickle.load(open(self._zscal_pickle_file, "r"))
                return
            except Exception as ex:
                self.managerlogger.logger.error("load zscal_pickle_file error: %s" % ex)
                self.errorlogger.logger.error("load zscal_pickle_file error:\n %s" % (traceback.format_exc()))
        self._scale = preprocessing.StandardScaler()

    def _store_zscore_scale(self):
        """Save scale result for pmml file, with MEAN and SCALE."""
        try:
            properties_path, properties_file = os.path.split(self._zscal_pickle_file)
            properties_file = os.path.splitext(properties_file)[0] + self.PROPERTIES
            preperties_file = os.path.join(properties_path, properties_file)
            # compine mean amd scale with column like "columnnA = {mean:3, scale:2}"
            res = map(
                lambda x: "%s = {\"%s\":%s, \"%s\":%s}" % (x[0], self.MEAN, x[1], self.SCALE, x[2]),
                zip(self._data.columns.tolist(), self._scale.mean_.tolist(),
                    self._scale.scale_.tolist()))
            with open(preperties_file, "w") as file:
                file.write("\n".join(res))
        except Exception as ex:
            self.managerlogger.logger.error("store properties error: %s" % (str(ex)))
            self.errorlogger.logger.error("store properties error:\n %s" % traceback.format_exc())

    def control_data(self, function_di):
        """
        Customized function. You can add customized functions here.

        Parameters
        ----------
        function_di: {"field":column, "function": function}
        """
        if function_di is None:
            self.managerlogger.logger.debug("processor function is None")
            return
        if len(function_di) == 0 and not isinstance(function_di, dict):
            return
        for key in function_di:
            try:
                tmp_field = key
                tmp_function = function_di[key]
                if tmp_field in self._data.columns.tolist():
                    self._data[tmp_field] = self._data[tmp_field].apply(tmp_function)

                    # unittest
                    # self._data[tmp_field] = self._data[tmp_field].apply(lambda x : x+x)

                    self.managerlogger.logger.info("process %s success" % (key))
            except Exception as ex:
                self.managerlogger.logger.error("function can not process %s" % (key))
                self.errorlogger.logger.error("function can not process %s" % (ex))



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import configparser
import unittest
import numpy as np
import pandas as pd
import sys

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import feature_filter


class TestFeatureFilter(unittest.TestCase):
    def setUp(self):
        # before each test case
        self.data_tt = pd.read_csv("data/featureFilter.csv")
        self.conf_1 = configparser.ConfigParser()
        self.conf = configparser.ConfigParser()
        self.ff_t = feature_filter.FeatureFilter(self.conf_1, log_path = '../log/log.conf')
        self.conf.read('./conf/feature_enginnering.conf')
        self.ff = feature_filter.FeatureFilter(self.conf, log_path = '../log/log.conf')
        self.ff_no = feature_filter.FeatureFilter('./conf/feature_enginnering.conf',log_path = '../log/log.conf')

    def test_start(self):
        self.assertTrue(self.ff.init())
        self.assertTrue(self.ff.set_data(data=self.data_tt))
        self.assertIsNotNone(self.ff.excute())

        self.assertFalse(self.ff_no.init())
        self.assertTrue(self.ff_no.set_data(data=self.data_tt))
        self.ff_no.excute()

        self.assertFalse(self.ff_t.init())
        self.ff_t.set_data(data=self.data_tt)
        self.ff_t.excute()

    def test_get_data(self):
        self.ff.set_data(data=self.data_tt)
        data = self.ff.get_data()
        self.assertIsNotNone(data)

    def test_set_data(self):
        data_no = None
        self.assertTrue(self.ff.init())
        self.assertTrue(self.ff.set_data(self.data_tt))
        data = self.ff.get_data()
        self.assertFalse(self.ff.set_data(data=data_no))
        self.assertIsNotNone(self.ff.excute())

    def test_label_fields(self):
        self.ff.set_data(self.data_tt)
        self.ff.label_fields()

    def test_store_label_properties(self):
        data_no = None
        self.ff._store_label_properties(data_no)

    def test_reset(self):
        self.ff.reset(data=self.data_tt, del_list=['col1'], del_keywords='col2',
                      label_list=np.random.randint(0, 2, 99),
                      selected_list=['col3', 'col4'])


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from ..systemlog import syserrorlog

class DataSet(object):
    def __init__(self):
        self._data = None

        self._id_fields = None
        self._target_field = None
        self._feature_fields = None

        self._data_id = None
        self._data_feature = None
        self._data_label = None

    def data_set_id(self):
        if self._data is None or self._id_fields is None:
            pass



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from . import xes_onehot_encoder
from . import feature_discretizer
from . import data_sampler
from . import data_washer
from . import feature_filter



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


from . import ml_utils

__all__ = ["predict_utils", "ml_utils", "data_sample", "data_wash", "feature_enginnering",
           "feature_filter",
           "model", "cross_validation",
           "store_data", "constance"]
